#ifndef INJducatimgr_h
#define INJducatimgr_h

#define INJ_INCR_RPM_MAPS_SIZE 8 // Engine Speed compensation map size
#define INJ_INCR_TIM_MAPS_SIZE 8 // Injection Time compensation map size
#define INJ_MAPS_TOTAL_NUM 2 // total number of maps
#define INJ_INCREMENT_RPM_STD 0 // standard value in case of EEPROM error
#define INJ_INCREMENT_TIM_STD 0 // standard value in case of EEPROM error

#define INJ_ENG_DATA_SD_COMM_BUFFER_SIZE 25 // Size of buffer for SD writing (Engine data only)
#define INJ_COMB_DATA_SD_COMM_BUFFER_SIZE 24 // Size of buffer for SD writing (Combustion data)
#define INJ_COMBUSTION_BUFFERS_NUM 5 // combustion buffer size

class INJducatimgr_class{
	
	public:
		void begin(); // Called in MAIN Setup
		uint16_t interpolate_tim_map(uint16_t inj_ticks_in);
		uint16_t interpolate_rpm_map(uint16_t inj_2r_ticks_in);
		uint8_t create_engine_data_packet(uint8_t* SD_writing_buffer, uint8_t max_length);
		void create_combustion_data_packet();
		
		// Variables for SD and COMM Service packet creation
		//uint8_t SD_writing_buffer[INJ_ENG_DATA_SD_COMM_BUFFER_SIZE]; // buffer for SD writing
		uint8_t SD_combustion_buffer[INJ_COMB_DATA_SD_COMM_BUFFER_SIZE]; // buffer for SD writing
		uint8_t packet_cnt = 0; // increasing counter
		
};

extern INJducatimgr_class INJducatimgr;

extern uint32_t Timer0_tick_counts();

// Variables to export
extern volatile uint16_t delta_time_tick[];
extern volatile uint16_t delta_inj_tick[];
extern volatile uint16_t extension_time_ticks[];
extern volatile uint8_t INJ_exec_time;
extern volatile uint16_t incrementi_tim[];
extern volatile uint16_t incrementi_rpm[];

extern volatile uint16_t injection_counter_buffer; // counts the combustion cycles
extern volatile uint16_t delta_time_tick_buffer; // time between 2 injections buffer
extern volatile uint16_t delta_inj_tick_buffer; // time of injection input buffer
extern volatile uint16_t extension_time_ticks_buffer; // extension time ticks
extern volatile uint8_t buffer_busy; // locks access to the buffer

// Combustion buffers
extern volatile uint8_t buffer_last_fill;
extern volatile uint8_t buffer_last_write;
//extern volatile uint8_t buffer_now_fill;
extern volatile uint8_t buffer_now_write;

#endif