#ifndef TEMPODOUBLE_cpp
#define TEMPODOUBLE_cpp

#include "tempodouble.h"

Tempodouble Timer1;

ISR(TIMER1_COMPA_vect)
{
  TIMSK1 &= ~_BV(OCIE1A); // clears the timer overflow interrupt enable bit, timer continues to count without calling the isr
  Timer1.isrCallbackA();
}

ISR(TIMER1_COMPB_vect)
{
  TIMSK1 &= ~_BV(OCIE1B); // clears the timer overflow interrupt enable bit, timer continues to count without calling the isr
  Timer1.isrCallbackB();
}


void Tempodouble::initialize()
{
  TCCR1A = 0; // clear control register A
  TCCR1B = 0; // turn on Normal mode
  clockSelectBits = _BV(CS11); // prescale by /8 = 0.5us at 16MHz
  //clockSelectBits = _BV(CS12) | _BV(CS10); // just for testing (1024 scaling)
}


// Attaches an interrupt to the Timer1 interrupt event
void Tempodouble::attachInterrupt(void (*isr)(), uint8_t num_int)
{
  if (!num_int){
	  isrCallbackA = isr;
  }else{
	  isrCallbackB = isr;
  }
}


// Starts the timer
void Tempodouble::start()
{ 
  TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
  TCCR1B |= clockSelectBits; // starts
}


// Stops the timer
void Tempodouble::stop()
{
  TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12)); // clears all clock selects bits
}


// Resets the counter
void Tempodouble::reset()
{ 
	oldSREG= SREG;
  	cli();	
	TCNT1H=0;
	TCNT1L=0;
	SREG = oldSREG;
}


// Return the number of Timer1 ticks
uint16_t Tempodouble::read(){				
	oldSREG= SREG;
  	cli();							
  	uint16_t tmp=TCNT1;    					
	SREG = oldSREG;
	return tmp;
}


void Tempodouble::settimer(uint16_t timer_value, uint8_t num_int){
	oldSREG= SREG;
	cli();
	uint16_t time_now_tmp = TCNT1;
	uint16_t time_stop_tmp = time_now_tmp + timer_value;
	if (!num_int){ // A
		OCR1A = time_stop_tmp; // sets threshold
		TIFR1 |= _BV(1);  // clears interrupt request flag (to avoid sudden execution)
		TIMSK1 |= _BV(OCIE1A); // activate interrupt
	}else{ // B
		OCR1B = time_stop_tmp; // sets threshold
		TIFR1 |= _BV(2);  // clears interrupt request flag (to avoid sudden execution)
		TIMSK1 |= _BV(OCIE1B); // activate interrupt
	}
	SREG = oldSREG;
}

#endif